{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "e405e6ff-5b00-4eee-b751-c28063e510f8",
   "metadata": {},
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "import pandas as pd\n",
    "import random\n",
    "import json\n",
    "import torch\n",
    "import torch.nn as nn\n",
    "from torch.utils.data import Dataset, DataLoader\n",
    "import nltk\n",
    "from nltk.stem.porter import PorterStemmer"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e1917503-1f5b-4d8f-846b-6b15fdce06ae",
   "metadata": {},
   "source": [
    "## NLTK UTILS"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "8d51a9bf-ea59-407b-9a2b-a5d46a89860a",
   "metadata": {},
   "outputs": [],
   "source": [
    "stemmer = PorterStemmer()\n",
    "def tokenize(sentence):\n",
    "    return nltk.word_tokenize(sentence)\n",
    "def stem(word):\n",
    "    return stemmer.stem(word.lower())\n",
    "def bag_of_words(tokenized_sentence, words):\n",
    "    sentence_words = [stem(word) for word in tokenized_sentence]\n",
    "    bag = np.zeros(len(words), dtype = np.float32)\n",
    "    for idx, w in enumerate(words):\n",
    "        if w in sentence_words:\n",
    "            bag[idx] = 1\n",
    "    return bag"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e485e808-897e-47c1-853c-e4c2ef50c85c",
   "metadata": {},
   "source": [
    "## Model"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "04ab27cf-4b92-4fb6-be3e-4e3ac6f9ca29",
   "metadata": {},
   "outputs": [],
   "source": [
    "class NeuralNet(nn.Module):\n",
    "    def __init__(self, input_size, hidden_size, num_classes):\n",
    "        super(NeuralNet, self).__init__()\n",
    "        self.l1 = nn.Linear(input_size, hidden_size)\n",
    "        self.l2 = nn.Linear(input_size, hidden_size)\n",
    "        self.l3 = nn.Linear(input_size, hidden_size)\n",
    "        self.output = nn.Linear(hidden_size, num_classes)\n",
    "        self.relu = nn.ReLU()\n",
    "    def forward(self, x):\n",
    "        out = self.l1(x)\n",
    "        out = self.relu(out)\n",
    "        out = self.l2(out)\n",
    "        out = self.relu(out)\n",
    "        out = self.l3(out)\n",
    "        out = self.output(out) #\n",
    "        return out"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "fe6a946f-982b-4e53-bfee-e9087214de71",
   "metadata": {},
   "outputs": [],
   "source": [
    "with open('intents.json', 'r', encoding='utf8') as f:\n",
    "    intents = json.load(f)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "c6f4e5d9-c00d-43d4-ab29-363037cc2e28",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{'intents': [{'tags': 'greeting',\n",
       "   'patterns': ['Hi',\n",
       "    'Hello',\n",
       "    'Namaste',\n",
       "    'Hey',\n",
       "    'How are you?',\n",
       "    'Are you there?',\n",
       "    'Is anyone here?'],\n",
       "   'responses': ['Hello, Thanks for Visiting DigiLocker',\n",
       "    'Namaste, Welcome to DigiSaathi',\n",
       "    'Hi DigiSaathi is here, How can I help?'],\n",
       "   'context_set': ''},\n",
       "  {'tags': 'goodbye',\n",
       "   'patterns': ['Bye', 'See you later', 'Goodbye'],\n",
       "   'responses': ['See you later', 'Thanks for Visiting', 'Have a nice Day']},\n",
       "  {'tags': 'thanks',\n",
       "   'patterns': ['Thanks', 'Thank you', \"That's helpful\"],\n",
       "   'responses': ['Happy to help!', 'Any time!', 'My pleasure']},\n",
       "  {'tags': 'access',\n",
       "   'patterns': ['What URLs can be used to access DigiLocker?'],\n",
       "   'responses': ['DigiLocker can be accessed at https://digilocker.gov.in'],\n",
       "   'context_set': ''},\n",
       "  {'tags': 'components',\n",
       "   'patterns': ['What are the key components of DigiLocker?'],\n",
       "   'responses': ['DigiLocker account has the following sections:\\n Home –  This is the home screen of your DigiLocker account, from where you can navigate to other sections of DigiLocker. This also shows summary of issued documents and a link to get documents from partners integrated with DigiLocker. \\n Issued Documents –  This section shows list of URIs (links) of digital documents or certificates issued by the Govt. departments or agencies integrated with DigiLocker. \\n DigiLocker Drive – This section shows all the documents which are uploaded by you. You can update the document type and share these uploaded documents. \\n Activity –  This section keeps log of all the activities performed by you in DigiLocker account. The log includes the details about activities such as file upload, download, share, etc. \\n Browse Documents –  This section provides the lists of departments and agencies that are registered with DigiLocker as Issuers. If these departments have issued any document/certificate to you, it will appear in the form of a URI (link) in your Issued Documents section.']},\n",
       "  {'tags': 'signup',\n",
       "   'patterns': ['How can I sign up for DigiLocker?'],\n",
       "   'responses': ['Signing up for DigiLocker is easy - all you need is your mobile or Aadhaar number. Your mobile/Aadhaar number will be authenticated by sending an OTP (one-time password) followed by setting your security PIN for 2-Factor authentication. This will create your DigiLocker account.']},\n",
       "  {'tags': 'updatemobile',\n",
       "   'patterns': ['My mobile is updated in my Aadhaar number, but I cannot use it to link my DigiLocker account with Aadhaar. It gives an error message - Mobile number is not linked with Aadhaar'],\n",
       "   'responses': ['For linking your DigiLocker account with Aadhaar, your mobile must be linked with your Aadhaar number. Please make sure this is actually the case. To verify this, visit https://eaadhaar.uidai.gov.in/  and download your eAadhaar. You will be able to check the linked mobile number (last four digits) during this process.']},\n",
       "  {'tags': 'ilnkaadhar',\n",
       "   'patterns': ['I wish to link Aadhaar with my DigiLocker, but my mobile number is not updated in my Aadhaar. How can I get this done?'],\n",
       "   'responses': ['You need to link Mobile number with Aadhaar. Please visit nearest Aadhaar Kendra. Click the link to get list of Aadhaar enrolment centres. https://uidai.gov.in/ecosystem/enrolment-ecosystem/aadhaar-seva-kendra.html']},\n",
       "  {'tags': 'issuer',\n",
       "   'patterns': ['My organization wants to utilize services of DigiLocker to become issuer /requester? How do we proceed?'],\n",
       "   'responses': ['To become Issuer/Requester of DigiLocker, your organization first needs to be registered with us. To know about the process of registration, please contact our customer support team at support[dot]digitallocker[dot]gov[dot]in. Your request will be forwarded to the DigiLocker on-boarding team and they’ll get back to you.']},\n",
       "  {'tags': 'marksheet',\n",
       "   'patterns': ['I am a CBSE student. How can I get my digital mark sheet on DigiLocker?'],\n",
       "   'responses': [\"How to Access Your CBSE Certificates: DigiLocker has created DigiLocker accounts of students using the mobile numbers shared by CBSE.Students have received an SMS on their mobile number registered with CBSE.Students should use this mobile number to login to their accounts.Enter the OTP received on registered Mobile NumberEnter the last 6 digits of your Roll number as the security PIN and Login.After successful Login, Student should go to 'Issued Document' section of DigiLocker and you will see all your class X or XII certificates here.For students' whose account could not be created (due to incorrect mobile numbers or any other reason), they need to sign up on DigiLocker and link their Aadhaar numbers. If they are not able to perform Aadhaar-OTP verification, Aadhaar can also be linked by entering demographic details (Aadhaar Number, Name, DoB and Gender as per Aadhaar). So even if Aadhaar is not linked to a correct mobile number, it still can be linked to DigiLocker account.After successful Login, Student should click on 'Browse' and select 'Central Board of Secondary Education' in Education category.Select the Document student is willing to fetch i.e. Marksheet, Passing Certificate or Migration Certificate.Enter the required details i.e. Year and Roll No and get your Class X or XII Certificates in your DigiLocker Account.\"]},\n",
       "  {'tags': 'driving',\n",
       "   'patterns': ['My name and DoB in Aadhaar is same as in DL and RC. I am still not able to fetch my DL/RC in my DigiLocker Account.'],\n",
       "   'responses': ['Please make sure that the DL/RC you are looking for is available on National Register. Check(1) For Driving License, please visit https://parivahan.gov.in/rcdlstatus/?pur_cd=101.(2) For RC, please visit https://vahan.parivahan.gov.in/vahanservice/vahan/ui/appl_status/form_Know_Appl_Status.xhtml.']},\n",
       "  {'tags': 'already',\n",
       "   'patterns': [\"I tried linking my Aadhaar with DigiLocker, but it gives an error 'This Aadhaar is already registered'. How can this be possible?\"],\n",
       "   'responses': ['This error can only be possible if your Aadhaar is already linked with a DigiLocker account and now you have created another DigiLocker account and trying to link Aadhaar with that account. We suggest you try to retrieve your credentials using Forgot Username/Password (using Aadhaar) for the first account and continue using it.']},\n",
       "  {'tags': 'issuer',\n",
       "   'patterns': [\"I am getting an error message 'The details provided did not match the issuer data. Please try again with some changes.' What does this mean and what can I do get my data?\"],\n",
       "   'responses': ['This message can come if some of the details filled by you are incorrect. Please re-enter the correct information and try again.']},\n",
       "  {'tags': 'license',\n",
       "   'patterns': [\"I followed the instructions but could not get my digital Driving License and Vehicle Registration Certificate via DigiLocker. It says 'No record available in issuer database for given document no. Please check your document number and try again.' Why I am not able to get my documents through DigiLocker?\"],\n",
       "   'responses': ['DigiLocker has integrated with the National Register, which is a central database maintained by the Ministry of Road Transport and Highways. If your Driving License and Vehicle Registration Certificate record does not exist in the National Register, DigiLocker will not be able to get it for you.']},\n",
       "  {'tags': 'license',\n",
       "   'patterns': ['Does my name matter while fetching my Digital Driving License and Vehicle Registration Certificate through DigiLocker? I can see a non-editable field for the Aadhaar name while filling out the requisite details.'],\n",
       "   'responses': ['While fetching the MoRTH digital records in DigiLocker, your name in your Aadhaar card should match your name in vehicle Registration Certificate & Driving License database of the National Register. This ensures that only the rightful owner of the documents is able to fetch the digital Driving License and Vehicle Registration Certificate.']},\n",
       "  {'tags': 'process',\n",
       "   'patterns': ['Please explain the process of getting the digital DL & RC in DigiLocker'],\n",
       "   'responses': ['For getting the digital Driving License and Vehicle Registration Certificate, users should ensure their Aadhaar number is linked with their DigiLocker account. Once it is done, User can go to the quick Link section on the dashboard and click on “Get your Digital Driving license” and “Get your Digital Vehicle RC”. This will allow to fetch document from the MoRTH database. Once the document is fetched, an URI is created and saved automatically in their ‘Issued section of DigiLocker’ section for later reuse.Citizens can get their Digital Driving License and Vehicle Registration Certificate on both desktop and on mobile devices.']},\n",
       "  {'tags': 'integration',\n",
       "   'patterns': ['Give us details about the DigiLocker integration with Ministry of Road Transport and Highways (MoRTH). What are the benefits of this integration for citizens?'],\n",
       "   'responses': ['DigiLocker is recognized by Ministry of Road Transport and Highways via notification (RT-11036/64/2017-MVL dated 08/08/2018) as a digital platform for making available digital driving license vehicle registration certificates to Indian citizens. DigiLocker is now directly integrated with the National Register, which is the national database of driving license and vehicle registration data across the country. Henceforth, DigiLocker users will be able to access their digital Registration Certificate Driving License both on desktop computers and on mobile devices.Benefits of this integration:Paperless Services: Digital driving license and vehicle registration certificate will minimize the use of physical documents.Authentic Records: Citizens can share the authentic digital certificates directly from the data source with other departments as identity and address proof resulting in reduction of administrative overhead.Spot Verification: The digital RC and DL in a DigiLocker account can be spot verified for authenticity either by validating the Digital Signature of MoRTH on the PDF copy of the document or by scanning the QR code on digital documents by using the QR scan facility on DigiLocker mobile app.No need to carry original physical form of documents while driving vehicle: Ministry of Road Transport and Highway have issued a D.O to all state transport states about acceptance of Driving License and Vehicle Registration Certificate when presented through issued section of DigiLocker are deemed to be legally recognized at par with the original documents as per the provisions of the Information Technology Act, 2000.']},\n",
       "  {'tags': 'aadhaar',\n",
       "   'patterns': ['I have already signed up for DigiLocker and linked my Aadhaar. How can I get my Digital Aadhaar in DigiLocker?'],\n",
       "   'responses': ['Here are the steps to get Digital Aadhaar in DigiLocker if you already have Aadhaar linked DigiLocker account:Login to DigiLocker with your credentials.Go to the issued section where you will see your digital Aadhaar Card listed.Just click on the view/download option to access the Digital Aadhaar.']},\n",
       "  {'tags': 'aadhaar',\n",
       "   'patterns': ['DigiLocker now allows citizens to access their digital Aadhaar. What is digital Aadhaar? Is it the same as the eAadhaar issued by UIDAI?'],\n",
       "   'responses': ['Digital Aadhaar in DigiLocker is the same as eAadhaar issued by UIDAI. DigiLocker has partnered with UIDAI to make it available automatically to its users when they link their DigiLocker account with Aadhaar.The advantage of digital Aadhaar is that it can be shared with any agency or organization in electronic form thus elimination use of Photocopies or print outs.']},\n",
       "  {'tags': 'files',\n",
       "   'patterns': ['What type of files can be uploaded?'],\n",
       "   'responses': ['File types that can be uploaded - pdf, jpeg & png.']},\n",
       "  {'tags': 'size',\n",
       "   'patterns': ['What is the maximum allowed file size that can be uploaded?'],\n",
       "   'responses': ['Maximum allowed file size is 10MB.']},\n",
       "  {'tags': 'upload',\n",
       "   'patterns': ['How can I upload documents to my DigiLocker account?'],\n",
       "   'responses': [\"You can upload documents from inside your ‘Uploaded Documents’ section.Click the upload icon to start uploading a document. In the file upload dialog box, locate the file from your local drive and select 'open' to complete the uploading.You can also edit the name of the file using the edit icon next to the filename.\"]},\n",
       "  {'tags': 'uri',\n",
       "   'patterns': ['What is the meaning of URI?'],\n",
       "   'responses': ['A URI is a Uniform Resource Identifier generated by the issuer department, which is mandatory for every e-document of the DigiLocker system. This unique URI can be resolved to a full URL (Uniform Resource Locator) to access the actual document in its appropriate repository.']},\n",
       "  {'tags': 'upload',\n",
       "   'patterns': ['What are issued documents and what are uploaded documents?'],\n",
       "   'responses': ['Issued documents are e-documents issued by various government agencies in electronic format directly from the original data source and the URI (link) of these documents is available in the issued documents section of DigiLocker. Whereas uploaded documents are those documents that are uploaded directly by the DigiLocker user.']},\n",
       "  {'tags': 'security',\n",
       "   'patterns': ['How to Reset Security PIN? (If the Old mobile number is lost)'],\n",
       "   'responses': [\"Try the following steps to reset the security PIN:(1) Kindly visit https://www.digilocker.gov.in/ and click on 'Sign In'.(2) Click on 'Forget Security PIN'.(3) Enter your Aadhaar number and DOB as per Aadhaar and click 'Submit'.(4) Go to the bottom of the page and click on 'Try using Aadhaar OTP instead'. This will send the OTP to your Aadhaar-registered mobile number.(5) Enter the OTP sent to you by UIDAI and Captcha text and click 'Submit'.(6) Set your new security PIN.Your new security PIN is set successfully\"]},\n",
       "  {'tags': 'updatemobile',\n",
       "   'patterns': ['How can I update my mobile number if I have lost my DigiLocker registered mobile number?'],\n",
       "   'responses': [\"Kindly try the following steps to log in via Aadhaar OTP and update your DigiLocker-linked mobile number:(1)Kindly visit https://www.digilocker.gov.in/ and click on 'Sign In'.(2) Enter your Aadhaar number and security PIN and then click 'Sign In'.(3) Go to the bottom of the page and click on 'Try using Aadhaar OTP instead!'. This will send the OTP to your Aadhaar-registered mobile number.(4) Enter the OTP sent to you by UIDAI and Captcha text and click 'Submit'.(5) You will be logged in successfully. After logging in, go to the profile section and edit your mobile number by clicking on the pencil icon beside a mobile.(6) Enter your new mobile number and click 'Submit'. OTP will be sent to your new mobile number.After successful verification, your DigiLocker mobile number will be updated.\"]},\n",
       "  {'tags': 'aadhaar',\n",
       "   'patterns': [\"I am frequently getting error 'Aadhaar already registered' during e-KYC.\"],\n",
       "   'responses': ['It seems that your Aadhaar number is already registered with DigiLocker and now you are trying to link Aadhaar again with your Non - Aadhaar account.Please try to login into your DigiLocker account by entering Aadhaar number. You will receive an OTP on your registered mobile number and submit the OTP.In case of any issue, please write to us with screenshot of error.']},\n",
       "  {'tags': 'username',\n",
       "   'patterns': ['How to update username?'],\n",
       "   'responses': ['Username once set during the Sign Up process cannot be changed. However, if you want to set up a username for the first time, please follow the following steps:(1) Update your DigiLocker App.(2) Login using your Aadhaar number, option to set your username (user alias) will appear. If you already have a username you will not get this option.(3) Username can contain dot (.), dash (-) and underscore (_).The username should be between 4-50 characters without space and must start with a small letter.(4) To know your username, kindly visit your profile after login.']},\n",
       "  {'tags': 'update',\n",
       "   'patterns': ['How to update name and DoB in DigiLocker account?'],\n",
       "   'responses': ['Please try the following options:(1) Make sure that the full name/DoB that you want in your Digilocker account is updated on your Aadhaar card.(2) Log In using your Aadhaar number and click on Browse--> Click on Aadhaar--> Click again to update Aadhaar card/Complete eKYC(3) Once you submit the OTP, the full name will be updated as per Aadhaar.(4) You will not be able to change name or DoB anything different than Aadhaar.In case of any issue write to us with screenshot of error and your Aadhaar/ Mobile number.Note: UIDAI usually take 20-30 days for the data updation.Please visit https://eaadhaar.uidai.gov.in/ and download your eAadhaar. This will tell you if your data is actually updated with Aadhaar or still in process.']},\n",
       "  {'tags': 'account',\n",
       "   'patterns': ['Can I open more than one DigiLocker Account with a single mobile no.?'],\n",
       "   'responses': ['You can open only one account with one mobile number which is not linked with Aadhaar. If you want to open more accounts with the same mobile number you will have to link your existing account with your Aadhaar number. Thereafter, you can use same mobile number to open another DigiLocker account by linking to Aadhaar.']},\n",
       "  {'tags': 'foreign',\n",
       "   'patterns': ['I am a NRI (Non Resident Indian), can I sign up using a foreign mobile number?'],\n",
       "   'responses': ['No, it is not possible. You can register in DigiLocker using Indian mobile number only.']},\n",
       "  {'tags': 'username',\n",
       "   'patterns': ['I have registered on DigiLocker and I want to change my username, is there any way to do so?'],\n",
       "   'responses': ['Username once created in DigiLocker cannot be changed currently.']},\n",
       "  {'tags': 'profile',\n",
       "   'patterns': ['How can I change the information displayed (like email, mobile etc) in my Aadhaar profile on DigiLocker?'],\n",
       "   'responses': ['The information displayed in your Aadhaar profile in your DigiLocker account (like name, address, email, mobile etc) is for display purposes. This data is only fetched from UIDAI and it is not possible to make any changes to this data from DigiLocker. To make changes to your Aadhaar data, please visit your nearest Aadhaar enrolment center.']},\n",
       "  {'tags': 'forgot',\n",
       "   'patterns': ['I have forgotten my DigiLocker username/password. What should I do?'],\n",
       "   'responses': ['You do not require username to login anymore. You can type your mobile number or Aadhaar instead of username and after providing the OTP, you will be able login to your account.']},\n",
       "  {'tags': 'message',\n",
       "   'patterns': [\"After entering the Aadhaar number, I am getting an error message 'UID service temporarily unavailable'.\"],\n",
       "   'responses': ['This may sometimes happen due to technical issues on our servers or with the UIDAI service. Most likely it’s a temporary problem. Please try again after sometime.']},\n",
       "  {'tags': 'otp',\n",
       "   'patterns': ['I am waiting for the OTP, but I have not received it for more than 5 minutes. What should I do?'],\n",
       "   'responses': ['While using DigiLocker, the OTP is sent instantaneously. Sometimes there may be temporary problems either on our servers (while generating/sending OTP) or in delivery of the SMS by your mobile service provider. We suggest you wait for few more minutes and try again.']},\n",
       "  {'tags': 'ilnkaadhar',\n",
       "   'patterns': ['While trying to link Aadhaar with DigiLocker, I am getting an error message Your Aadhaar is invalid.'],\n",
       "   'responses': ['For registering with DigiLocker, please make sure you have correctly entered the 12 digit Aadhaar number that you received from UIDAI in the given text box.']},\n",
       "  {'tags': 'ilnkaadhar',\n",
       "   'patterns': ['I wish to link Aadhaar with my DigiLocker, but my mobile number is not updated in my Aadhaar. How can I get this done?'],\n",
       "   'responses': ['You need to link Mobile number with Aadhaar. Please visit nearest Aadhaar Kendra. Click the link to get list of Aadhaar enrolment centres.https://uidai.gov.in/ecosystem/enrolment-ecosystem/aadhaar-seva-kendra.html']},\n",
       "  {'tags': 'ilnkaadhar',\n",
       "   'patterns': ['My mobile is updated in my Aadhaar number, but I cannot use it to link my DigiLocker account with Aadhaar. It gives an error message - Mobile number is not linked with Aadhaar'],\n",
       "   'responses': ['For linking your DigiLocker account with Aadhaar, your mobile must be linked with your Aadhaar number. Please make sure this is actually the case.To verify this, visit https://eaadhaar.uidai.gov.in/ and download your eAadhaar. You will be able to check the linked mobile number (last four digits) during this process.']},\n",
       "  {'tags': 'signup',\n",
       "   'patterns': ['How can I sign up for DigiLocker?'],\n",
       "   'responses': ['Signing up for DigiLocker is easy - all you need is your mobile or Aadhaar number. Your mobile/Aadhaar number will be authenticated by sending an OTP (one-time password) followed by setting your security PIN for 2-Factor authentication. This will create your DigiLocker account.']}]}"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "intents"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "id": "99fde2cb-60e8-445a-b87a-87e6be4124ea",
   "metadata": {},
   "outputs": [],
   "source": [
    "all_words = []\n",
    "tags = []\n",
    "xy = []"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "id": "982cb118-42a8-4ab5-8768-1e9575e83e7c",
   "metadata": {},
   "outputs": [],
   "source": [
    "for i in intents['intents']:\n",
    "    tag = i['tags']\n",
    "    tags.append(tag)\n",
    "    for j in i['patterns']:\n",
    "        w = tokenize(j)\n",
    "        all_words.extend(w)\n",
    "        xy.append((w, tag))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "id": "b49b1837-dfd3-4664-856f-713f25f3b9c8",
   "metadata": {},
   "outputs": [],
   "source": [
    "ignore_words = ['?','.','&',\"'\",'!','(',')',',','-']\n",
    "all_words = [stem(w) for w in all_words if w not in ignore_words]\n",
    "all_words = sorted(set(all_words))\n",
    "tags = sorted(set(tags))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "id": "2a4a3fc8-5ca2-41ae-83a9-754e253d8086",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "50 Patterns\n",
      "29 Tags: ['aadhaar', 'access', 'account', 'already', 'components', 'driving', 'files', 'foreign', 'forgot', 'goodbye', 'greeting', 'ilnkaadhar', 'integration', 'issuer', 'license', 'marksheet', 'message', 'otp', 'process', 'profile', 'security', 'signup', 'size', 'thanks', 'update', 'updatemobile', 'upload', 'uri', 'username']\n",
      "199 Unique Stemmed Words: [\"'aadhaar\", \"'no\", \"'s\", \"'the\", \"'thi\", \"'uid\", '/request', '5', 'a', 'aadhaar', 'abl', 'about', 'access', 'account', 'after', 'again', 'allow', 'alreadi', 'am', 'an', 'and', 'ani', 'anyon', 'are', 'as', 'avail', 'be', 'becom', 'benefit', 'but', 'by', 'bye', 'can', 'cbse', 'certif', 'chang', 'check', 'citizen', 'compon', 'could', 'data', 'databas', 'detail', 'did', 'digilock', 'digit', 'display', 'dl', 'dl/rc', 'do', 'dob', 'document', 'doe', 'done', 'drive', 'dure', 'e-kyc', 'eaadhaar', 'email', 'enter', 'error', 'etc', 'explain', 'fetch', 'field', 'file', 'fill', 'follow', 'for', 'foreign', 'forgotten', 'frequent', 'get', 'give', 'given', 'goodby', 'have', 'hello', 'help', 'here', 'hey', 'hi', 'highway', 'how', 'i', 'if', 'in', 'indian', 'inform', 'instruct', 'integr', 'invalid', 'is', 'issu', 'issuer', 'it', 'key', 'later', 'licens', 'like', 'link', 'lost', 'mark', 'match', 'matter', 'maximum', 'mean', 'messag', 'ministri', 'minut', 'mobil', 'more', 'morth', 'my', 'namast', 'name', 'no', 'non', 'non-edit', 'not', 'now', 'nri', 'number', 'of', 'old', 'on', 'one', 'open', 'organ', 'otp', 'out', 'pin', 'pleas', 'possibl', 'proceed', 'process', 'profil', 'provid', 'rc', 'receiv', 'record', 'regist', 'registr', 'requisit', 'reset', 'resid', 'road', 'same', 'say', 'secur', 'see', 'servic', 'sheet', 'should', 'sign', 'singl', 'size', 'so', 'some', 'still', 'student', 'temporarili', 'than', 'thank', 'that', 'the', 'their', 'there', 'thi', 'through', 'to', 'transport', 'tri', 'type', 'uidai', 'unavail', 'up', 'updat', 'upload', 'uri', 'url', 'us', 'use', 'usernam', 'username/password', 'util', 'vehicl', 'via', 'wait', 'want', 'way', 'we', 'what', 'whi', 'while', 'wish', 'with', 'you', 'your']\n"
     ]
    }
   ],
   "source": [
    "print(len(xy), \"Patterns\")\n",
    "print(len(tags), \"Tags:\", tags)\n",
    "print(len(all_words), \"Unique Stemmed Words:\", all_words)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "id": "6bfcd615-d7bf-4ce9-89e0-4ab13c41ad3d",
   "metadata": {},
   "outputs": [],
   "source": [
    "x_train = []\n",
    "y_train = []\n",
    "for (pattern_sentence, tag) in xy:\n",
    "    bag = bag_of_words(pattern_sentence, all_words)\n",
    "    x_train.append(bag)\n",
    "    label = tags.index(tag)\n",
    "    y_train.append(label)\n",
    "x_train = np.array(x_train)\n",
    "y_train = np.array(y_train)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "id": "49761452-cae7-47a5-815b-16b87a921f8e",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "199 29\n"
     ]
    }
   ],
   "source": [
    "num_epochs = 1000\n",
    "batch_size = 8\n",
    "learning_rate = 0.0002\n",
    "input_size = len(x_train[0])\n",
    "hidden_size = 199\n",
    "output_size = len(tags)\n",
    "print(input_size, output_size)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "id": "10988e27-76e8-409b-a811-4c8df7d1dcdd",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Epoch [100/1000], Loss: 0.0980\n",
      "Epoch [200/1000], Loss: 0.0013\n",
      "Epoch [300/1000], Loss: 0.0009\n",
      "Epoch [400/1000], Loss: 0.0007\n",
      "Epoch [500/1000], Loss: 0.0002\n",
      "Epoch [600/1000], Loss: 0.0002\n",
      "Epoch [700/1000], Loss: 0.3743\n",
      "Epoch [800/1000], Loss: 0.0002\n",
      "Epoch [900/1000], Loss: 0.0017\n",
      "Epoch [1000/1000], Loss: 0.0001\n",
      "Final Loss: 0.0001\n"
     ]
    }
   ],
   "source": [
    "class ChatDataset(Dataset):\n",
    "    def __init__(self):\n",
    "        self.n_samples = len(x_train)\n",
    "        self.x_data = x_train\n",
    "        self.y_data = y_train\n",
    "    def __getitem__(self, index):\n",
    "        return self.x_data[index], self.y_data[index]\n",
    "    def __len__(self):\n",
    "        return self.n_samples\n",
    "\n",
    "dataset = ChatDataset()\n",
    "train_loader = DataLoader(dataset=dataset, batch_size=batch_size, shuffle=True, num_workers=0)\n",
    "device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')\n",
    "model = NeuralNet(input_size, hidden_size, output_size).to(device)\n",
    "\n",
    "criterion = nn.CrossEntropyLoss()\n",
    "optimizer = torch.optim.Adam(model.parameters(), lr= learning_rate)\n",
    "\n",
    "for epoch in range(num_epochs):\n",
    "    for (words, labels) in train_loader:\n",
    "        words = words.to(device)\n",
    "        labels = labels.to(dtype=torch.long).to(device)\n",
    "        outputs = model(words)\n",
    "        loss= criterion(outputs, labels)\n",
    "        optimizer.zero_grad()\n",
    "        loss.backward()\n",
    "        optimizer.step()\n",
    "\n",
    "    if(epoch+1)%100 == 0:\n",
    "        print(f'Epoch [{epoch+1}/{num_epochs}], Loss: {loss.item():.4f}')\n",
    "print(f'Final Loss: {loss.item():.4f}')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "id": "5bebc804-ed07-4de4-bbab-8e8d8b066880",
   "metadata": {},
   "outputs": [],
   "source": [
    "data= {\n",
    "    'model_state':model.state_dict(),\n",
    "    'input_size':input_size,\n",
    "    'hidden_size':hidden_size,\n",
    "    'output_size':output_size,\n",
    "    'all_words':all_words,\n",
    "    'tags':tags\n",
    "}"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 19,
   "id": "82ff4ab9-1f95-40a0-87df-8407415fe752",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Training Complete. File Saved to data.pth\n"
     ]
    }
   ],
   "source": [
    "FILE = 'data.pth'\n",
    "torch.save(data, FILE)\n",
    "print(f'Training Complete. File Saved to {FILE}')"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "879f80f2-0241-4c2b-8c5f-ec725c61de6c",
   "metadata": {},
   "source": [
    "## CHAT"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 20,
   "id": "2a09a9c2-a9d2-4018-b507-2220760490aa",
   "metadata": {},
   "outputs": [],
   "source": [
    "data = torch.load(FILE)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 21,
   "id": "07def208-0b28-45f6-b03a-80941195cabb",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "OrderedDict([('l1.weight',\n",
       "              tensor([[-0.0595, -0.0791, -0.0498,  ..., -0.0142,  0.0017,  0.0341],\n",
       "                      [ 0.0151,  0.0171, -0.0126,  ...,  0.0571,  0.0771, -0.0210],\n",
       "                      [ 0.0464,  0.0465, -0.0368,  ...,  0.0185, -0.0010,  0.0290],\n",
       "                      ...,\n",
       "                      [-0.0595,  0.0401, -0.0220,  ..., -0.0087, -0.0917,  0.1449],\n",
       "                      [-0.0447, -0.0594, -0.0077,  ..., -0.0419,  0.0027, -0.0273],\n",
       "                      [ 0.0052, -0.0240, -0.0009,  ..., -0.0252, -0.0161, -0.0339]])),\n",
       "             ('l1.bias',\n",
       "              tensor([ 1.2294e-01,  1.4685e-01,  4.8549e-02,  1.5489e-01,  1.0826e-01,\n",
       "                       1.0325e-01, -1.2933e-02, -2.4994e-02,  1.4274e-01,  4.9784e-02,\n",
       "                       1.0406e-01,  3.3593e-02,  1.2537e-01,  1.3319e-01,  9.2162e-02,\n",
       "                       1.7593e-02,  3.4063e-02,  2.3401e-02,  5.9898e-02,  1.5092e-01,\n",
       "                       7.5954e-02,  8.6649e-02,  1.3953e-01,  3.8904e-02,  6.6539e-02,\n",
       "                       1.8214e-02,  9.0844e-02, -2.7883e-02,  8.4681e-02, -9.9535e-03,\n",
       "                       1.4368e-02,  9.4338e-02,  1.1073e-01,  6.6576e-02,  5.7056e-02,\n",
       "                       2.9333e-02,  1.3573e-01,  1.5030e-01,  7.0769e-02,  9.0577e-02,\n",
       "                       7.5426e-02,  1.2803e-01,  1.2255e-01,  8.9056e-03,  5.5851e-02,\n",
       "                      -6.1838e-03,  1.4380e-01, -1.7309e-02,  1.6514e-01,  3.3599e-02,\n",
       "                       7.4056e-02,  1.2338e-01, -2.1932e-02,  4.0725e-02,  7.4254e-02,\n",
       "                       7.1944e-02,  6.3712e-02,  6.5188e-02,  1.1801e-01,  2.8939e-02,\n",
       "                      -3.4573e-02,  1.4617e-01,  7.5877e-02,  3.0374e-02,  1.3196e-01,\n",
       "                       4.4957e-02,  8.1433e-02, -1.7545e-02,  7.9777e-03,  5.5361e-04,\n",
       "                       2.2278e-02,  1.3995e-01,  1.1340e-01, -1.2837e-02,  1.5251e-01,\n",
       "                      -1.2813e-02,  6.9566e-02,  1.4188e-01,  2.3695e-02,  1.3403e-01,\n",
       "                       8.4945e-03, -2.1276e-02,  8.0225e-02,  8.0488e-02, -9.4557e-03,\n",
       "                       7.1961e-02,  1.1801e-01,  4.4723e-02,  7.4673e-02,  8.6880e-02,\n",
       "                       7.4266e-03,  1.4594e-01,  5.0039e-02,  7.3996e-02,  3.7044e-02,\n",
       "                       9.4088e-02,  2.2753e-02,  7.6766e-02,  3.8055e-02, -9.5916e-03,\n",
       "                      -1.2157e-02,  1.4840e-01,  9.0016e-02,  9.9716e-02,  1.5188e-01,\n",
       "                       1.0514e-01,  2.9543e-03, -7.2086e-03,  4.9603e-02,  1.5818e-01,\n",
       "                       1.3349e-01, -3.6999e-02,  1.0830e-01,  1.4513e-01,  1.2380e-01,\n",
       "                       4.0244e-02,  1.5991e-01,  3.0463e-02,  1.5567e-02,  4.9100e-02,\n",
       "                       2.2258e-02, -6.9583e-04,  1.1420e-01,  1.4964e-01,  1.5324e-01,\n",
       "                       5.7216e-02, -1.1639e-02, -2.2791e-02,  1.5177e-01,  1.5181e-01,\n",
       "                      -1.4484e-02,  7.8911e-02,  3.2015e-02,  1.0717e-01,  9.6514e-02,\n",
       "                       6.7562e-02,  4.3702e-02,  1.5123e-01, -2.3822e-02,  1.3376e-01,\n",
       "                       1.0825e-01,  6.7556e-02,  1.3427e-01,  6.0353e-02, -1.3258e-02,\n",
       "                       8.9677e-03,  7.7769e-02,  4.4229e-02,  8.1042e-02,  1.0507e-01,\n",
       "                       1.6777e-01,  2.1470e-02,  1.1014e-01,  7.2021e-02,  1.4210e-02,\n",
       "                       1.4154e-01,  2.2967e-02,  1.2005e-01,  1.2894e-01,  9.0165e-02,\n",
       "                       5.4165e-02,  4.2643e-02,  3.4105e-02,  4.6441e-02,  1.1973e-01,\n",
       "                       6.2286e-02, -1.0338e-02,  1.2707e-01,  1.3898e-01,  1.3491e-01,\n",
       "                      -2.9385e-02,  1.6649e-01, -1.4289e-02, -9.6661e-03,  2.5388e-02,\n",
       "                       1.5063e-01,  8.2111e-02,  6.9168e-02, -6.6417e-04,  3.9133e-02,\n",
       "                       1.5744e-02,  1.3383e-01,  1.6680e-01,  1.1150e-01, -1.9535e-02,\n",
       "                       9.0788e-03,  1.2171e-01,  5.8815e-02, -1.0370e-02,  8.5561e-02,\n",
       "                       6.2763e-02,  6.9091e-03,  7.4692e-03, -2.5611e-02, -3.0908e-02,\n",
       "                       2.0841e-02,  1.1704e-04,  9.2620e-02,  1.1366e-01])),\n",
       "             ('l2.weight',\n",
       "              tensor([[ 0.0336,  0.0410,  0.0620,  ...,  0.0617,  0.0494, -0.0693],\n",
       "                      [-0.0691, -0.0661,  0.0205,  ..., -0.0622,  0.0505,  0.0704],\n",
       "                      [ 0.1262,  0.0666,  0.0042,  ...,  0.0325,  0.1385,  0.0459],\n",
       "                      ...,\n",
       "                      [ 0.1025,  0.0864, -0.0347,  ...,  0.0129, -0.1078, -0.0540],\n",
       "                      [ 0.0130, -0.0071,  0.0732,  ...,  0.0658,  0.0376,  0.0206],\n",
       "                      [ 0.0479, -0.0678,  0.0226,  ..., -0.0050, -0.0225, -0.0559]])),\n",
       "             ('l2.bias',\n",
       "              tensor([-0.0476,  0.0889, -0.0056,  0.0452, -0.0178, -0.0270,  0.0656,  0.0710,\n",
       "                       0.0978, -0.0332, -0.0116,  0.0318, -0.0123, -0.0235,  0.0156, -0.0621,\n",
       "                       0.0448, -0.0738, -0.0215,  0.0611,  0.0472, -0.0026,  0.0164,  0.0450,\n",
       "                       0.0184, -0.0126,  0.0775,  0.0216,  0.0020, -0.0438, -0.0789,  0.1204,\n",
       "                       0.0749, -0.0363,  0.1287, -0.0551,  0.0721, -0.0368,  0.0286,  0.0320,\n",
       "                      -0.0563, -0.0302,  0.0996,  0.0926, -0.0598,  0.0844, -0.0548, -0.0240,\n",
       "                      -0.0416,  0.1193,  0.0597,  0.0758,  0.0816, -0.0364,  0.1154,  0.0690,\n",
       "                       0.0869, -0.0124,  0.1156, -0.0404, -0.0338,  0.0908, -0.0008,  0.0576,\n",
       "                      -0.0690,  0.1076, -0.0036,  0.0884, -0.0620, -0.0705,  0.1201,  0.1137,\n",
       "                      -0.0475,  0.0439,  0.0650, -0.0394, -0.0467,  0.0638,  0.0981,  0.0402,\n",
       "                      -0.0078,  0.0290,  0.0518,  0.0227,  0.0147, -0.0369, -0.0375, -0.0184,\n",
       "                       0.0899, -0.0478, -0.0120,  0.0373,  0.0850,  0.0299,  0.0493,  0.0047,\n",
       "                      -0.0434, -0.0120,  0.0325, -0.0663,  0.0632, -0.0279,  0.0267, -0.0228,\n",
       "                      -0.0547,  0.0852,  0.0253,  0.0621,  0.0807,  0.0919, -0.0427,  0.0620,\n",
       "                      -0.0530, -0.0537,  0.0760,  0.0393,  0.0380, -0.0037,  0.0773,  0.0005,\n",
       "                       0.0527,  0.0863,  0.1025,  0.0928,  0.0930, -0.0658, -0.0369,  0.0283,\n",
       "                       0.0991,  0.0278,  0.0388, -0.0364, -0.0320, -0.0438, -0.0023,  0.0408,\n",
       "                       0.0486,  0.1084, -0.0435, -0.0012, -0.0368, -0.0157, -0.0717, -0.0348,\n",
       "                      -0.0652, -0.0228, -0.0083, -0.0496,  0.0180,  0.0339, -0.0180,  0.0032,\n",
       "                      -0.0144,  0.0354,  0.0137,  0.0848, -0.0438,  0.0224,  0.0094, -0.0338,\n",
       "                       0.0638,  0.1305, -0.0320,  0.0580,  0.1070, -0.0665,  0.0100,  0.0087,\n",
       "                       0.0445,  0.0869,  0.1110,  0.1073, -0.0344,  0.0887,  0.0501, -0.0343,\n",
       "                      -0.0245, -0.0546, -0.0192,  0.0611,  0.0612, -0.0284,  0.0468, -0.0388,\n",
       "                      -0.0550,  0.0319, -0.0242,  0.0512,  0.0177,  0.1192, -0.0023, -0.0314,\n",
       "                       0.0600,  0.0342,  0.0546,  0.0897,  0.0063,  0.0334,  0.0615])),\n",
       "             ('l3.weight',\n",
       "              tensor([[ 0.0585, -0.1368,  0.0060,  ...,  0.0212,  0.0029, -0.0880],\n",
       "                      [ 0.0287, -0.0054, -0.0055,  ...,  0.0436, -0.1328,  0.0581],\n",
       "                      [ 0.1023, -0.0168,  0.0130,  ...,  0.0828, -0.0567, -0.0212],\n",
       "                      ...,\n",
       "                      [ 0.0050,  0.0310, -0.0087,  ...,  0.0606, -0.0550, -0.1041],\n",
       "                      [ 0.0812, -0.0308, -0.1263,  ...,  0.0467, -0.0722, -0.1034],\n",
       "                      [ 0.0512, -0.0189,  0.0312,  ...,  0.0076,  0.0101,  0.0203]])),\n",
       "             ('l3.bias',\n",
       "              tensor([-0.0336, -0.0179, -0.0407, -0.0419, -0.0114, -0.0622, -0.0671, -0.0166,\n",
       "                      -0.0667, -0.0234,  0.0077, -0.0157,  0.0354, -0.0423, -0.0185, -0.0640,\n",
       "                       0.0740, -0.0472, -0.0371, -0.0894,  0.0616,  0.0346, -0.0437, -0.0359,\n",
       "                      -0.0036,  0.0317, -0.0791, -0.0236, -0.0291, -0.0635, -0.0380, -0.0673,\n",
       "                      -0.0328, -0.0215, -0.0055,  0.0450,  0.0382, -0.0392, -0.0471,  0.0222,\n",
       "                       0.0365, -0.0139, -0.0370, -0.0322,  0.0666,  0.0709, -0.0136,  0.0574,\n",
       "                       0.0768,  0.0073,  0.0702, -0.0456, -0.0616, -0.0515,  0.0372,  0.0472,\n",
       "                       0.0694,  0.0440, -0.0517, -0.0363,  0.0719,  0.0665, -0.0521, -0.0457,\n",
       "                      -0.0552,  0.0382, -0.0550, -0.0855, -0.0271, -0.0643, -0.0179,  0.0073,\n",
       "                      -0.0875,  0.0330,  0.0588, -0.0365,  0.0188,  0.0119, -0.0555, -0.0523,\n",
       "                      -0.0029,  0.0375,  0.0524, -0.0694, -0.0562,  0.0230,  0.0466,  0.0071,\n",
       "                      -0.0412,  0.0287, -0.0551,  0.0082, -0.0235,  0.0424,  0.0513,  0.0110,\n",
       "                       0.0173, -0.0357,  0.0282, -0.0314, -0.0191, -0.0168,  0.0857, -0.0348,\n",
       "                      -0.0613,  0.0518, -0.0369, -0.0394,  0.0265,  0.0737, -0.0376, -0.0317,\n",
       "                      -0.0836, -0.0371,  0.0052,  0.0629,  0.0124,  0.0519,  0.0596,  0.0508,\n",
       "                      -0.0235,  0.0725,  0.0288,  0.0129, -0.0808,  0.0664,  0.0136, -0.0355,\n",
       "                       0.0238,  0.0672, -0.0833,  0.0091,  0.0143, -0.0808, -0.0601,  0.0280,\n",
       "                      -0.0382, -0.0526,  0.0384,  0.0830, -0.0721,  0.0446, -0.0164, -0.0607,\n",
       "                      -0.0394,  0.0948, -0.0245,  0.0196,  0.0158,  0.0074,  0.0480,  0.0103,\n",
       "                      -0.0582, -0.0607, -0.0327, -0.0293, -0.0173,  0.0829,  0.0361,  0.0346,\n",
       "                       0.0365, -0.0580, -0.0213,  0.0299, -0.0364, -0.0255,  0.0268,  0.0011,\n",
       "                       0.0702,  0.0359,  0.0695,  0.0356,  0.0504, -0.0663,  0.0170,  0.0536,\n",
       "                       0.0026, -0.0472,  0.0121, -0.0179, -0.0248, -0.0053,  0.0493, -0.0914,\n",
       "                      -0.0551,  0.0155, -0.0222,  0.0506,  0.0108,  0.0371, -0.0148,  0.0336,\n",
       "                      -0.0265,  0.0327, -0.0506,  0.0398, -0.0543,  0.0255,  0.0522])),\n",
       "             ('output.weight',\n",
       "              tensor([[-0.0654, -0.0164, -0.1150,  ...,  0.0769,  0.1309, -0.0951],\n",
       "                      [ 0.0103,  0.1108,  0.0555,  ..., -0.0882, -0.1274,  0.1404],\n",
       "                      [-0.1413, -0.0877, -0.0873,  ...,  0.0922, -0.1020, -0.0278],\n",
       "                      ...,\n",
       "                      [ 0.0622,  0.0041,  0.1072,  ..., -0.0824, -0.0620,  0.1204],\n",
       "                      [-0.0532,  0.0222,  0.0703,  ...,  0.0563,  0.0631,  0.0653],\n",
       "                      [-0.1127,  0.1439,  0.1145,  ..., -0.0853, -0.0502,  0.0489]])),\n",
       "             ('output.bias',\n",
       "              tensor([-0.0081,  0.0375, -0.0281,  0.0336, -0.0100,  0.0476, -0.0062,  0.0461,\n",
       "                      -0.0363,  0.0257,  0.0262,  0.0484, -0.0765, -0.0122, -0.0374, -0.0119,\n",
       "                      -0.0740, -0.0585, -0.0523,  0.0075, -0.0621, -0.0011,  0.0137,  0.0750,\n",
       "                      -0.0628, -0.0285, -0.0303,  0.0002,  0.0204]))])"
      ]
     },
     "execution_count": 21,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "model_state = data['model_state']\n",
    "model_state"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 22,
   "id": "14fdbba1-049e-4a00-8706-4e740b1e4dfc",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "NeuralNet(\n",
       "  (l1): Linear(in_features=199, out_features=199, bias=True)\n",
       "  (l2): Linear(in_features=199, out_features=199, bias=True)\n",
       "  (l3): Linear(in_features=199, out_features=199, bias=True)\n",
       "  (output): Linear(in_features=199, out_features=29, bias=True)\n",
       "  (relu): ReLU()\n",
       ")"
      ]
     },
     "execution_count": 22,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "model = NeuralNet(input_size, hidden_size, output_size).to(device)\n",
    "model.load_state_dict(model_state)\n",
    "model.eval()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 23,
   "id": "a1a95921-1a3c-45f2-800b-d7f4aedee002",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Hi! Welcome to DigiSaathi.\n"
     ]
    },
    {
     "name": "stdin",
     "output_type": "stream",
     "text": [
      "You:  quit\n"
     ]
    }
   ],
   "source": [
    "bot_name = \"DigiSaathi\"\n",
    "print('Hi! Welcome to DigiSaathi.')\n",
    "# print('Press 1 to get issued documents.')\n",
    "# print('Press 2 to get chat support')\n",
    "while True:\n",
    "    sentence = input(\"You: \")\n",
    "    if(sentence == \"quit\"):\n",
    "        break\n",
    "    sentence = tokenize(sentence)\n",
    "    # print(f'Tokenized-{sentence}')\n",
    "    X = bag_of_words(sentence, all_words)\n",
    "    # if(1 in X):\n",
    "    #     print('huihui')\n",
    "    X = X.reshape(1, X.shape[0])\n",
    "    X = torch.from_numpy(X).to(device)\n",
    "    # print(f'X pt2-{X}')\n",
    "    output = model(X)\n",
    "    # print(f'OUTPUT-{output}')\n",
    "    _, predicted = torch.max(output, dim=1)\n",
    "    # print(f'PREDICTED- {predicted}')\n",
    "    tag = tags[predicted.item()]\n",
    "    # print(f'TAG-{tag}')\n",
    "    probs = torch.softmax(output, dim=1)\n",
    "    prob = probs[0][predicted.item()]\n",
    "    # print(f'prob-{prob}')\n",
    "    if prob.item() > 0.75:\n",
    "        # print(prob.item())\n",
    "        # print(f'tag:{tag}')\n",
    "        for intent in intents['intents']:\n",
    "            if tag == intent['tags']:\n",
    "                print(f'{bot_name}: {random.choice(intent['responses'])}')\n",
    "                break\n",
    "    else:\n",
    "        # print(prob.item())\n",
    "        # for intent in intents['intents']:\n",
    "        #     if tag == intent['tags']:\n",
    "        #         print(f'Error Response: {random.choice(intent['responses'])}')\n",
    "        print(f'{bot_name}: Sorry, I am not sure if I got that. Could you please try rephrasing your question?')\n",
    "    print('-------------------------------------------------------------------------------------')"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.0"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
